@@ -1,12 +1,15 @@
 """ Module to ETL data to generate pipelines """
 from __future__ import print_function
+import random
+import copy
 import asyncio
 import logging
 from pygyver.etl.dw import read_sql 
 from pygyver.etl.lib import extract_args
 from pygyver.etl.dw import BigQueryExecutor
 from pygyver.etl.toolkit import read_yaml_file
-
+from pygyver.etl.lib import add_dataset_id_prefix
+from pygyver.etl.lib import bq_default_project
 
 
 def async_run(func):
@@ -42,16 +45,59 @@ async def execute_parallel(func, args, message='running task', log=''):
     return len(count)
 
 
+def extract_unit_test_value(unit_test_list):     
+    utl = copy.deepcopy(unit_test_list)
+    for d in utl:           
+        file = d.pop('file')
+        d["sql"] = read_sql(file=file, **d)
+        d["cte"] = read_sql(file=d['mock_file'], **d)      
+        d["file"] = file     
+    return utl
+
+
+def extract_unit_tests(batch_list=None):
+    """ return the list of unit test: unit test -> file, mock_file, output_table_name(opt) """
+
+    # initiate args and argsmock
+    args, args_mock = [] , []
+
+    # extracts files paths for unit tests 
+    for batch in batch_list:            
+        for table in batch.get('tables', ''):
+            if table.get('create_table', '') != '' and table.get('mock_data', ''):                
+                args.append(table.get('create_table', ''))
+                args_mock.append(table.get('mock_data', ''))
+    
+    return_list = []
+    for a, b in zip(args, args_mock):
+        a.update(b)                        
+    return args
+    
+
 class PipelineExecutor:
-    def __init__(self, yaml_file):
+    def __init__(self, yaml_file, dry_run=False):
         self.yaml = read_yaml_file(yaml_file)
+        self.dry_run_dataset_prefix = None
+        if dry_run:
+            self.dry_run_dataset_prefix = random.sample(range(1,1000000000),1)[0]
+            add_dataset_id_prefix(self.yaml, self.dry_run_dataset_prefix)
         self.bq = BigQueryExecutor()
-        self.unit_test_list = self.extract_unit_tests()
+        # self.unit_test_list = extract_unit_tests()
         self.prod_project_id = 'copper-actor-127213'
+    
+    def dry_run_clean(self):
+        if self.dry_run_dataset_prefix is not None:
+            if bq_default_project() != self.prod_project_id:
+                for table in self.yaml.get('table_list', ''):
+                    if self.bq.dataset_exists(dataset_id= str(self.dry_run_dataset_prefix) + "_" + table.split(".")[0]):
+                        self.bq.delete_dataset(dataset_id=str(self.dry_run_dataset_prefix) + "_" + table.split(".")[0], delete_contents=True)
 
     def create_tables(self, batch):
+        args = [] # initiate args
         batch_content = batch.get('tables', '')
         args = extract_args(batch_content, 'create_table')
+        for a in args:
+            a.update({"dry_run_dataset_prefix": self.dry_run_dataset_prefix})
         if args != []:            
             result = execute_parallel(
                         self.bq.create_table,
@@ -62,6 +108,7 @@ class PipelineExecutor:
             return result
             
     def load_google_sheets(self, batch):
+        args = [] # initiate args
         batch_content = batch.get('sheets', '')
         args = extract_args(batch_content, 'load_google_sheet')
         if args == []:
@@ -75,10 +122,13 @@ class PipelineExecutor:
         return result
 
     def run_checks(self, batch):
+        args, args_pk = [] , [] # initiate args
         batch_content = batch.get('tables', '')
-        args = extract_args(batch_content, 'create_table')
+        args = extract_args(batch_content, 'create_table') # adding create_table args to args
+        for a in args: # adding dry_run_dataset_prefix to args
+            a.update({"dry_run_dataset_prefix": self.dry_run_dataset_prefix})
         args_pk = extract_args(batch_content, 'pk')
-        for a, b in zip(args, args_pk):
+        for a, b in zip(args, args_pk): # adding pk args to args
             a.update({"primary_key": b})
         result = execute_parallel(
                     self.bq.assert_unique,
@@ -107,39 +157,11 @@ class PipelineExecutor:
             self.run_batch(batch)
         # run release (ToDo)
 
-    def extract_unit_tests(self, batch_list=None):
-        """ return the list of unit test: unit test -> file, mock_file, output_table_name(opt) """
-        # extract sql files and mock data
+    def run_unit_tests(self, batch_list=None):
         batch_list = batch_list or self.yaml.get('batches', '')
-
-        # initiate args and argsmock
-        args, args_mock = [] , []
-
-        # extracts files paths for unit tests 
-        for batch in batch_list:            
-            for table in batch.get('tables', ''):
-                if table.get('create_table', '') != '' and table.get('mock_data', ''):                
-                    args.append(table.get('create_table', ''))
-                    args_mock.append(table.get('mock_data', ''))
-        
-        return_list = []
-        for a, b in zip(args, args_mock):
-            a.update(b)                        
-        return args
-        
-    def extract_unit_test_value(self, unit_test_list):                
-        for d in unit_test_list:
-            file = d.pop('file')
-            d["sql"] = read_sql(file=file, **d)
-            d["cte"] = read_sql(file=d['mock_file'], **d)      
-            d["file"] = file     
-        return unit_test_list
-
-    def run_unit_tests(self, yaml_content=None):
-        yaml_content = yaml_content or self.yaml
         # extract unit tests
-        list_unit_test = self.extract_unit_tests()
-        args = self.extract_unit_test_value(list_unit_test)
+        list_unit_test = extract_unit_tests(batch_list)
+        args = extract_unit_test_value(list_unit_test)
         if args != []:            
             result = execute_parallel(
                         self.bq.assert_acceptance,
@@ -160,7 +182,7 @@ class PipelineExecutor:
                     "source_project_id" : self.prod_project_id,
                     "source_dataset_id" : table.split(".")[0], 
                     "source_table_id": table.split(".")[1],
-                    "dest_dataset_id" : table.split(".")[0], 
+                    "dest_dataset_id" : str(self.dry_run_dataset_prefix) + "_" + table.split(".")[0], 
                     "dest_table_id": table.split(".")[1]
                 }                                    
             )            
